struct prm {

  /* This structure contains all of the globally visible information
     concerning runtime CCM Parameters set by the users either in the
     code itself, as command line instructions, using the GUI, or in
     some input file. 
     
     This is distinct from Control Parameters which simply tell the 
     code how much output to make, what filenames to use. 
     
  */


  int n;                // SUBn-m (n) 
  int m;                // SUBn-m (m) 
  int btLength;
  int dim;              // Dimension Of Lattice
  int nFundG;           // # Fund. Configs G-State
  int nFundE;           // # Fund. Configs E-State
  float s;              // Global Quantum Spin Number
  int ddd;

};

extern int  max_number_cells_BT;

struct ctrl {
  /* 
     
     Control Parameters which simply tell the 
     code how much output to make, what filenames to use. 
     
  */

  int output;           // Level Of Output
  int file;             // save CCM equations to file
  char name[60];        // Root File Name                

};

extern prm Params;

extern ctrl Control;

struct UC {

  /* this structure completely defines the unit cell */

  int number;             // number of sites in unit cell (UC)
  double x[MAX_UNIT_CELL];   // number of sites in UC
  double y[MAX_UNIT_CELL];   // number of sites in 
  double z[MAX_UNIT_CELL];   // number of sites in 
  float s[MAX_UNIT_CELL]; // `local' quantum spin number for each site in UC
  
};

extern UC UnitCell;

struct neighbours {

  /* this is an interesting structure because it defines the 
     nearest neighbour structure for each of the elements of 
     the unit cell.

     Essentially each unit cell has however many nearest-neighbours,
     some of which in within the unit cell itself, and some which
     are in neigbouring cells.

     This information is most easily coded in by the user, or
     by writing a simple bit of code to determine the neighbours
     matrix of the lattice or Hamiltonian.

     Note that this also offers the possiblity of defining
     nearest-neighbours, next-nearest-neighbours, next-next-...
     and so on.

  */

  int number[MAX_UNIT_CELL]; /* number of nearest-neighbours for
				each site in the UC */
  int x[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION x bravais
  int y[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION y bravais
  int z[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION z bravais
  int cell[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // which site in UC
  
};

extern neighbours LatticeNeighbours_k_to_m;
extern neighbours LatticeNeighbours_m_to_k;
extern neighbours LatticeNeighbours;

struct bravais {

  /* structure defines the bavais lattice */

  double a[4];
  double b[4];
  double c[4];

};

extern bravais BravaisLattice;

struct symmetries {

  /* this structure contains all of the information concerning
     the symmetries of the lattice, Hamiltonian, unit cell etc. */

  int number;     // number of symmetries in lattice, Hamiltonian etc
  int switchList[MAX_NUM_SYM]; // which symmetries from function switch(int)
  int switchFunction(int switchVariable, 
		     double x1, 
		     double y1, 
		     double z1,
		     double &x2, 
		     double &y2,
		     double &z2);
};

extern symmetries LatticeSymmetries;

struct spinConfigs {
  
  /* 

     This structure describes a particular spin configuration by storing 
     the information in an array. The size of the (integer) numbers
     becomes too long for saving as a standard `int' -- or even `long int.'
     
     Each integer element gives the information on 20 lattices sites
     each of which has a spin of quantum spin number Params.s or
     UnitCell.s[]
     
  */
  
  int descriptor[MAX_SPIN_LIST];
  
};

extern spinConfigs spinConfigurations[MAX_NUM_CONFIGURATIONS];
extern spinConfigs connectedConfigurations[MAX_NUM_CONFIGURATIONS];
extern int         NumberConfigurations;

extern double CCMcoeffs[MAX_NUM_CONFIGURATIONS];


struct hamiltonian_coefficients{
  double coefficient; 
};

extern hamiltonian_coefficients Hamiltonian[60];

extern int lookUpTable[MAX_BT+1][5];
extern int BTarray[MAX_LSUBM][MAX_LSUBM][MAX_LSUBM];

extern double x_array[100000];
extern double y_array[100000];
extern double z_array[100000];
extern int    x_index[100000];
extern int    y_index[100000];
extern int    z_index[100000];
extern int    cell_index[100000];
extern int    totalPoints;
