

struct prm {

  /* This structure contains all of the globally visible information
     concerning runtime CCM Parameters set by the users either in the
     code itself, as command line instructions, using the GUI, or in
     some input file. 
     
     This is distinct from Control Parameters which simply tell the 
     code how much output to make, what filenames to use. 
     
  */


  int n;                // SUBn-m (n) 
  int m;                // SUBn-m (m) 
  int btLength;         // Determines size of Bounding Traingle
  int dim;              // Dimension Of Lattice
  int nFundG;           // # Fund. Configs G-State
  int nFundE;           // # Fund. Configs E-State
  float s;              // Global Quantum Spin Number
  // NOTE Params.s MUST be set as big as the largest .s in the UNIT CELL!!!
  int ddd;
  int bonds;            // number of `bonds'

};
prm Params;

struct ctrl {
  /* 
     
     Control Parameters which simply tell the 
     code how much output to make, what filenames to use. 
     
  */

  int output;           // Level Of Output
  int file;             // save CCM equations to file
  char name[200];        // Root File Name                

};
ctrl Control;

struct UC {

  /* this structure completely defines the unit cell */

  int number;             // number of sites in unit cell (UC)
  double x[MAX_UNIT_CELL];   // 
  double y[MAX_UNIT_CELL];   // number of sites in 
  double z[MAX_UNIT_CELL];   // number of sites in 
  float s[MAX_UNIT_CELL]; // `local' quantum spin number for each site in UC
  float s2[MAX_UNIT_CELL]; // `local' quantum spin number for each site in UC

};
UC UnitCell;

struct sub {

  /* which terms from the unit cell to determine the
     sublattice for */

  int number;
  int cell[MAX_UNIT_CELL];

};

sub sublatticeMagnetisation;

struct neighbours {

  /* this is an interesting structure because it defines the 
     nearest neighbour structure for each of the elements of 
     the unit cell.

     Essentially each unit cell has however many nearest-neighbours,
     some of which in within the unit cell itself, and some which
     are in neigbouring cells.

     This information is most easily coded in by the user, or
     by writing a simple bit of code to determine the neighbours
     matrix of the lattice or Hamiltonian.

     Note that this also offers the possiblity of defining
     nearest-neighbours, next-nearest-neighbours, next-next-...
     and so on.

  */

  int number[MAX_UNIT_CELL]; /* number of nearest-neighbours for
				each site in the UC */
  int x[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION x bravais
  int y[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION y bravais
  int z[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // TRANSLATION z bravais
  int cell[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS]; // which site in UC
  
};
neighbours LatticeNeighbours;

struct bravais {

  /* structure defines the bavais lattice */

  double a[4];
  double b[4];
  double c[4];

};
bravais BravaisLattice;

struct symmetries {

  /* this structure contains all of the information concerning
     the symmetries of the lattice, Hamiltonian, unit cell etc. */

  int number;     // number of symmetries in lattice, Hamiltonian etc

  int switchFunction(int switchVariable, 
		     double x1, 
		     double y1, 
		     double z1,
		     double &x2, 
		     double &y2,
		     double &z2);

  double R[MAX_NUM_SYM][4][4]; // ROTATION MATRIX

  double x[MAX_NUM_SYM];       // NON-BRAVAIS LATTICE VECTOR.
  double y[MAX_NUM_SYM];
  double z[MAX_NUM_SYM];
  

};
symmetries LatticeSymmetries;

struct spinConfigs {
  
  /* 

     This structure describes a particular spin configuration by storing 
     the information in an array. The size of the (integer) numbers
     becomes too long for saving as a standard `int' -- or even `long int.'
     
     Each integer element gives the information on 20 lattices sites
     each of which has a spin of quantum spin number Params.s or
     UnitCell.s[]
     
  */
  
  int descriptor[MAX_SPIN_LIST+3];
  
};

spinConfigs spinConfigurations[MAX_NUM_CONFIGURATIONS];
spinConfigs connectedConfigurations[MAX_NUM_CONFIGURATIONS];
spinConfigs latticeAnimals[MAX_NUM_CONFIGURATIONS];
int         NumberConfigurations;
int         NumberLatticeAnimals;

double CCMcoeffs[MAX_NUM_CONFIGURATIONS]={0.0};

struct hamiltonian_defn{

  int evaluate[200]; 
  double coefficient[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][200]; 

  int k_to_m_number[MAX_UNIT_CELL][200];                
  int k_to_m_x[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][200]; 
  int k_to_m_y[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][200]; 
  int k_to_m_z[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][200]; 
  int k_to_m_cell[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][200];

  int m_to_k_number[MAX_UNIT_CELL][200];                
  int m_to_k_x[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][200]; 
  int m_to_k_y[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][200]; 
  int m_to_k_z[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][200]; 
  int m_to_k_cell[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][200]; 
};

hamiltonian_defn Hamiltonian[3];

struct type_defn{

  int evaluate[200]; 
  double coefficient[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][17]; 

  int k_to_m_number[MAX_UNIT_CELL][17];                
  int k_to_m_x[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][17]; 
  int k_to_m_y[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][17]; 
  int k_to_m_z[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][17]; 
  int k_to_m_cell[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][17];

  int m_to_k_number[MAX_UNIT_CELL][17];                
  int m_to_k_x[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][17]; 
  int m_to_k_y[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][17]; 
  int m_to_k_z[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][17]; 
  int m_to_k_cell[MAX_UNIT_CELL][MAX_NUM_NEIGHBOURS][17]; 
};

type_defn Hamiltonian_types[3];

// Other global variables.

int  max_number_cells_BT;
int lookUpTable[MAX_BT+1][5];
int BTarray[MAX_LSUBM][MAX_LSUBM][MAX_LSUBM];

double x_array[2];
double y_array[2];
double z_array[2];
int    x_index[2];
int    y_index[2];
int    z_index[2];
int    cell_index[2];
int    totalPoints;

/* obtain */

int binary_max[31];
int trimary_max[MAX_LSUBM+1];
int quartary_max[MAX_LSUBM+1];

// file pointers.

FILE *fp;
FILE *fp_gs;
FILE *fp1;
FILE *fp2;

// solve variables.

double maximum;
double minimum;
double increment;
double delta;

// restriction on choice of clusters.

int restrictionClusters;

// direction bonds or not?

int directional;

// restriction of sweeping for F_k F_m s_k^z s_m^z terms.

int midPointX;
int midPointY;
int midPointZ;
int radius;

// solution strategy

int solutionStrategy;

// parallel variables

int myid;
int npe;
